@@ -25,5 +25,7 @@ void lttng_ust_unlock_fd_tracker(void);
 int lttng_ust_safe_close_fd(int fd, int (*close_cb)(int));
 int lttng_ust_safe_fclose_stream(FILE *stream, int (*fclose_cb)(FILE *stream));
 int lttng_ust_safe_closefrom_fd(int lowfd, int (*close_cb)(int));
+int lttng_ust_safe_close_range_fd(unsigned int first, unsigned int last, int flags,
+		int (*close_range_cb)(unsigned int, unsigned int, int));
 
 #endif	/* _LTTNG_UST_FD_H */
